/* File Name: BVCApp.cpp */

/*
************************************************************************
************************************************************************

Copyright © 1998 Multimedia Integration Products, All Rights Reserved

This file is for public release.

This file is for reference use only. Any redistribution must include
the entire BeVC package, including source code. If you use this code
please notify j-onorato@uchicago.edu or jwhamil@students.wisc.edu

************************************************************************

************************************************************************
************************************************************************ */


#include "BVCApp.h"
#include "main.h"

// *********************************************//************
BVCApp::BVCApp()
	: BApplication("application/x-vnd.MIP-BeVC")
{
/* Load the prefs */
	const void	* data;
	size_t		out_size;
	uint32		out_type;
	char str[B_PATH_NAME_LENGTH];

	prefsObject = new Preferences("x-mip-bvc");
	prefsSet = new PreferenceSet(*prefsObject, "settings", true);
	
/* Get User Name */
	if (prefsSet->InitCheck() == B_OK &&
				prefsSet->GetData("userName", data, out_size, out_type) == B_OK &&
				out_type == B_STRING_TYPE)
		strncpy(userName, (char *) data, B_OS_NAME_LENGTH);
	else
		strncpy(userName, "", B_OS_NAME_LENGTH);
	userName[B_OS_NAME_LENGTH - 1] = '\0';
	
/* Get Password */
	if (prefsSet->InitCheck() == B_OK &&
				prefsSet->GetData("password", data, out_size, out_type) == B_OK &&
				out_type == B_STRING_TYPE)
		strncpy(password, (char *) data, B_OS_NAME_LENGTH);
	else
		strncpy(password, "", B_OS_NAME_LENGTH);
	userName[B_OS_NAME_LENGTH -1] = '\0';
	
/* Get Repository Directory */
	if (prefsSet->InitCheck() == B_OK &&
				prefsSet->GetData("repositoryPath", data, out_size, out_type) == B_OK &&
				out_type == B_STRING_TYPE)
		strncpy(str, (char *) data, B_PATH_NAME_LENGTH);
	else
		strncpy(str, "/boot/home", B_PATH_NAME_LENGTH);
	
	repositoryPath.SetTo(str);
	userName[B_PATH_NAME_LENGTH -1] = '\0';
	
/* Get Repository Directory */
	if (prefsSet->InitCheck() == B_OK &&
				prefsSet->GetData("workDirPath", data, out_size, out_type) == B_OK &&
				out_type == B_STRING_TYPE)
		strncpy(str, (char *) data, B_PATH_NAME_LENGTH);
	else
		strncpy(str, "/boot/home/work", B_PATH_NAME_LENGTH);
	userName[B_PATH_NAME_LENGTH -1] = '\0';
	
	workDirPath.SetTo(str);
	
/* Get Empty Repository after Checkin */
	if (prefsSet->InitCheck() == B_OK &&
				prefsSet->GetData("deleteAfterCheckin", data, out_size, out_type) == B_OK &&
				out_type == B_BOOL_TYPE)
		deleteAfterCheckin = *(bool *) data;
	else
		deleteAfterCheckin = false;
	
/* Get Repository Type (Local/Remote) */
	if (prefsSet->InitCheck() == B_OK &&
				prefsSet->GetData("remoteRepository", data, out_size, out_type) == B_OK &&
				out_type == B_BOOL_TYPE)
		remoteRepository = *(bool *) data;
	else
		remoteRepository = false;
		
/* Get Remote Repository Address */
	if (prefsSet->InitCheck() == B_OK &&
				prefsSet->GetData("serverAddress", data, out_size, out_type) == B_OK &&
				out_type == B_STRING_TYPE)
		strncpy(serverAddress, (char *) data, 100);
	else
		strncpy(serverAddress, "127.0.0.1", 100);
	userName[100 - 1] = '\0';
	
	workDirPath.SetTo(str);
	
/* Start the repository going */
//Check for local/remote later	
	repository = new RepositoryLooper();
	
/* Open the Window and go from there */
	scWindow = new SourceControlWindow();

/* Make the passwd window - but don't show it */
	chPasswdWindow = new ChangePasswordWindow();
	
}


// *********************************************//************
BVCApp::~BVCApp()
{
	delete prefsSet;
	delete prefsObject;
}


// *********************************************//************
void BVCApp::MessageReceived(BMessage * message)
{
	status_t err;
	
	switch (message->what)
	{
		case MIP_VC_GET_MODULES:
		case MIP_VC_CHECK_IN:
		case MIP_VC_UPDATE:
		case MIP_VC_CHECK_OUT:
		case MIP_VC_MODS_OUT_LIST:
			 err = repository->PostMessage(message);
			 break;
	
		case VC_RELOAD_MODULE_LIST:
		case MIP_VC_CHECK_IN_REPLY:
		case MIP_VC_UPDATE_REPLY:
		case MIP_VC_CHECK_OUT_REPLY:
		case MIP_VC_MODS_LIST:
		case MIP_VC_MODS_OUT_LIST_REPLY:
		case MIP_VC_USER_PASS_ERROR:
		case MIP_VC_AUTH_FAILED:
			err = scWindow->PostMessage(message);
			break;
		
		case VC_SHOW_CHANGE_PASSWD_WINDOW:
		{
			if(chPasswdWindow->IsHidden())
				chPasswdWindow->Show();
			chPasswdWindow->Activate(true);
		}
		break;
		
		default:
			inherited::MessageReceived(message);
	}
}


// *********************************************//************
bool BVCApp::QuitRequested()
{
/* Save the prefs */
	char str[B_PATH_NAME_LENGTH];

	prefsSet->SetData("userName", &userName, strlen(userName)+1, B_STRING_TYPE);
	prefsSet->SetData("password", &password, strlen(password)+1, B_STRING_TYPE);
	
	prefsSet->SetData("deleteAfterCheckin", &deleteAfterCheckin, sizeof(deleteAfterCheckin),	B_BOOL_TYPE);
	prefsSet->SetData("remoteRepository", &remoteRepository, sizeof(remoteRepository),	B_BOOL_TYPE);

	strncpy(str, serverAddress, 100);
	prefsSet->SetData("serverAddress", &str, strlen(str)+1, B_STRING_TYPE);
	
	strncpy(str, workDirPath.Path(), B_PATH_NAME_LENGTH);
	prefsSet->SetData("workDirPath", &str, strlen(str)+1, B_STRING_TYPE);

	strncpy(str, repositoryPath.Path(), B_PATH_NAME_LENGTH);
	prefsSet->SetData("repositoryPath", &str, strlen(str)+1, B_STRING_TYPE);

	prefsSet->Save();

	return(true);
}
