/* File Name: RepositoryLooper.cpp */

/*
************************************************************************
************************************************************************

Copyright © 1998 Multimedia Integration Products, All Rights Reserved

This file is not for public release.
************************************************************************

************************************************************************
************************************************************************ */


#include "RepositoryLooper.h"
#include "main.h"
#include "SourceControlWindow.h"
#include "NetMessenger.h"
#include "SecurityManager.h"

// *********************************************//************
RepositoryLooper::RepositoryLooper()
	: BLooper("RepositoryLooper")
{
	Run();
}


// *********************************************//************
RepositoryLooper::~RepositoryLooper()
{
	
}


// *********************************************//************
void RepositoryLooper::MessageReceived(BMessage * message)
{
	status_t	err;
	
	remoteConnection = message->IsSourceRemote();
	
	switch (message->what)
	{
		case MIP_VC_GET_MODULES:
		{
			if (!AuthentUser(message))
				return;
			
			SendModulesList();
		}
		break;
		
		case MIP_VC_CHECK_IN:
		{
			if (!AuthentUser(message))
				return;
				
			if (!reposLock)
			{
				reposLock = true;
				SaveModule(message);
				SendModulesList();

				reposLock = false;
			}
			else
			{
				BMessage temp(MIP_VC_REPOS_BUSY);
				SendToClient(&temp);
			}
		}
		break;
	
		case MIP_VC_UPDATE:
		{
			if (!AuthentUser(message))
				return;

			char	* userNameTemp;
			message->FindString("user", &userNameTemp);

			Update(userNameTemp);
		}
		break;

		case MIP_VC_CHECK_OUT:
		{
			
			if (!AuthentUser(message))
				return;
			
			if (!reposLock)
			{
				reposLock = true;
				CheckOut(message);

				char	* userNameTemp;
				message->FindString("user", &userNameTemp);
				Update(userNameTemp);
				SendModulesList();
				
				reposLock = false;
			}
			else
			{
				BMessage temp(MIP_VC_REPOS_BUSY);
				SendToClient(&temp);
			}
		}
		break;		
		
		case MIP_VC_MODS_OUT_LIST:				// Get a list of mods out for 'user' with 'passwd'
		{
			if (!AuthentUser(message))
				return;
			
			char	* userNameTemp;
			message->FindString("user", &userNameTemp);
			
			BMessage reply(MIP_VC_CHECK_IN_REPLY);
			GetModsOutList(userNameTemp, &reply);
			
			SendToClient(&reply);
		}
		break;
		
		default:
			inherited::MessageReceived(message);
	}
}


// *********************************************//************
bool RepositoryLooper::QuitRequested()
{
	return true;
}


// *********************************************//************
bool RepositoryLooper::AuthentUser(BMessage * message)
{
	if (!remoteConnection)						// If we're local don't deal with preferences
		return true;
	
	status_t err;
	
	char	* userNameTemp,
			* passwordTemp;
	
	err = message->FindString("user", &userNameTemp);
	if (err != B_OK)
	{
		SendToClient(&BMessage(MIP_VC_USER_PASS_ERROR));
		return false;
	}
	
	err = message->FindString("passwd", &passwordTemp);
	if (err != B_OK)
	{
		SendToClient(&BMessage(MIP_VC_USER_PASS_ERROR));
		return false;
	}
	
	bool authenticated;
	authenticated = ::AuthenticateUser(userNameTemp, passwordTemp);
	if (!authenticated)
	{
		SendToClient(&BMessage(MIP_VC_AUTH_FAILED));
		return false;
	}
	
	return true;
}



// *********************************************//************
void RepositoryLooper::SendModulesList()
{
	status_t err;
	BMessage reply(MIP_VC_MODS_LIST);
	err = MakeModulesList(&reply);
	
	if (err == B_OK)
		SendToClient(&reply);
	else
		;										// Send back an error reply
	
	if (remoteConnection)
		be_app->PostMessage(&reply);
}


// *********************************************//************
// Fill the BMessage with modules and their information
status_t RepositoryLooper::MakeModulesList(BMessage * modList)
{
	status_t	err;
	BPath		modulesDirPath(repositoryPath);
	modulesDirPath.Append("modules");
	
	BDirectory	modulesDir(modulesDirPath.Path());
	entry_ref	modulesRef;
	
	modList->MakeEmpty();
	
	modList->what = MIP_VC_MODS_LIST;

	err	= modulesDir.InitCheck();
	modulesDir.Rewind();
	while (modulesDir.GetNextRef(&modulesRef) == B_OK)
	{
		if (strstr(modulesRef.name, ".modInfo") != NULL)
		{
			BPath	moduleInfoFilePath(modulesDirPath);
			moduleInfoFilePath.Append(modulesRef.name);
			
			FILE	* moduleInfoFile = fopen(moduleInfoFilePath.Path(), "rb");
			
			char	verNoTemp[B_OS_NAME_LENGTH];
			char	statusTemp[B_OS_NAME_LENGTH];
			char	userNameTemp[B_OS_NAME_LENGTH];
			time_t	lastTimeTemp;
			
			fscanf(moduleInfoFile, "%s", verNoTemp);
			fscanf(moduleInfoFile, "%s", statusTemp);
			fscanf(moduleInfoFile, "%s", userNameTemp);
//			fscanf(moduleInfoFile, "%d", lastTimeTemp);
			
			fclose(moduleInfoFile);

			ModuleItem * modItem;
			
			char	workModName[B_FILE_NAME_LENGTH];
			strcpy(workModName, modulesRef.name);
			char	* zipExt;
			zipExt = strstr(workModName, ".zip");
			if (zipExt != NULL)
			{
				int endIndex = zipExt - workModName;
				workModName[endIndex] = '\0';
			}
			
			char timeString[B_OS_NAME_LENGTH];
			strcpy(timeString, ctime(&lastTimeTemp));
			timeString[strlen(timeString) - 1] = '\0';

			BMessage modInfo(VC_MODS_INFO);
			if (!modInfo.IsEmpty())				//Make sure it's clean
				modInfo.MakeEmpty();
			modInfo.AddString("ver", verNoTemp);
			modInfo.AddString("name", workModName);
			modInfo.AddString("status", statusTemp);
			modInfo.AddString("who", userNameTemp);
			modInfo.AddString("time", B_EMPTY_STRING);
			
												//Add this mod to the list of them all
			modList->AddMessage("module", &modInfo); 
		}
	}
	
	return B_OK;

}


// *********************************************//************
status_t RepositoryLooper::SaveModule(const BMessage * msg)
// This take a BMessages with a zipped module to check in
// and handles saving it away in the backups dir and the modules dir
{
	int32	numMods;
	char	* buf, * name;
	ssize_t	size;
	status_t err;
	type_code	typeCode;
												//How many are in there?
	err = msg->GetInfo("module", &typeCode, &numMods);
		
	for (int i=0; i<numMods; i++)
	{
		msg->FindData("module", B_ANY_TYPE, i, &buf, &size);
		msg->FindString("name", i, &name);
		err = SaveModule(name, buf, size);
	}
	
												// Now send the new set of modules to the client
	SendModulesList();
}


// *********************************************//************
status_t RepositoryLooper::SaveModule(const char * moduleName, const char * buffer, ssize_t size)
{
	status_t err;

												// Where the modules go
	BPath	modulesDirPath(repositoryPath);
	modulesDirPath.Append("modules");

/* Get the new version string */
	char	oldVersionString[B_OS_NAME_LENGTH];
	char	newVersionString[B_OS_NAME_LENGTH];
	
	GetVersion(moduleName, oldVersionString);
	IncrementVersionNumber(oldVersionString, newVersionString);
	
/* Make the .modInfo file */
	char	modInfoFileName[B_PATH_NAME_LENGTH];
	sprintf(modInfoFileName, "%s/%s.zip.modInfo", modulesDirPath.Path(), moduleName);
	
	FILE * modInfoFile = fopen(modInfoFileName, "wb");
	
	time_t theTime = real_time_clock();
	
	fprintf(modInfoFile, "%s\n", newVersionString);
	fprintf(modInfoFile, "%s\n", "in");
	fprintf(modInfoFile, "%s\n", userName);
	fprintf(modInfoFile, "%d\n", theTime);
	
	fclose(modInfoFile);
	
/* Save the copy in the modules dir */
	char	modFilePathName[B_PATH_NAME_LENGTH];
	sprintf(modFilePathName, "%s/%s.zip", modulesDirPath.Path(), moduleName);
	
	MakeFileFromBuffer(buffer, modFilePathName, size);


												// Where the backups go
	BPath	backupDirPath(repositoryPath);
	backupDirPath.Append("backups");

/* Make the .bakInfo File */
	char	bakInfoFileName[B_PATH_NAME_LENGTH];
	sprintf(bakInfoFileName, "%s/%s-%s.bakInfo", backupDirPath.Path(), moduleName, newVersionString);
	FILE * bakInfo = fopen(bakInfoFileName, "wb");
	
	BDirectory savedModulesDir(modulesDirPath.Path());
	
	entry_ref	modulesRef;
//	char		modInfoFileName[B_PATH_NAME_LENGTH];
	char		versionString[B_OS_NAME_LENGTH];
	
	err	= savedModulesDir.InitCheck();
	savedModulesDir.Rewind();
	while (savedModulesDir.GetNextRef(&modulesRef) == B_OK)
	{
		if (strstr(modulesRef.name, ".modInfo") == NULL)
		{
			sprintf(modInfoFileName, "%s/%s.modInfo", modulesDirPath.Path(), modulesRef.name);
			
			FILE * modInfoFile = fopen(modInfoFileName, "rb");
			fscanf(modInfoFile, "%s", versionString);		// The first line of the file is the version string
			fclose(modInfoFile);
			
			fprintf(bakInfo, "%s\t%s\n", versionString, modulesRef.name);
		}
	}
	fclose(bakInfo);
	
/* Save the Backup */
	char	backupFilePathName[B_PATH_NAME_LENGTH];
	sprintf(backupFilePathName, "%s/%s-%s.zip", backupDirPath.Path(), moduleName, newVersionString);
	
	MakeFileFromBuffer(buffer, backupFilePathName, size);
	
}


// *********************************************//************
void RepositoryLooper::GetVersion(const char * moduleName, char * versionString)
{
/* Where the modules go */
	BPath	modulesDirPath(repositoryPath);
	modulesDirPath.Append("modules");
	
	char modInfoLeafName[B_FILE_NAME_LENGTH];
	sprintf(modInfoLeafName, "%s.zip.modInfo", moduleName);
	
	modulesDirPath.Append(modInfoLeafName);
	
	FILE * modInfoFile = fopen(modulesDirPath.Path(), "r");
	fscanf(modInfoFile, "%s", versionString);		// The first line of the file is the version string
	fclose(modInfoFile);
}


// ********************************************//************
void RepositoryLooper::IncrementVersionNumber(const char * oldVer, char * newVer)
{
	if (oldVer == NULL || strcmp(oldVer, "") == 0)
		strcpy(newVer, "0.1");
	else
	{
		char * lastPart;
		lastPart = strchr(oldVer, '.') + 1;
		if(lastPart == NULL)
			lastPart = (char *) oldVer;
		
		int	preVerLength = lastPart - oldVer;
		
		int postVer = atoi(lastPart);
		
		++postVer;
		
		memcpy(newVer, oldVer, preVerLength);
		newVer[preVerLength] = '\0';
		
		sprintf(newVer, "%s%d", newVer, postVer);
	}

}


// ********************************************//************
status_t RepositoryLooper::GetModsOutList(const char * user, BMessage * msg)
{
	status_t	err;
	entry_ref	modulesRef;

/* Where the modules live */
	BPath		modulesDirPath(repositoryPath);
	modulesDirPath.Append("modules");
	
	BDirectory	modulesDir(modulesDirPath.Path());
	err	= modulesDir.InitCheck();
	if (err != B_OK)
		return err;
	
/* Go through each file in the modules dir, checking to see if it is a .modInfo file,
   if it is, check to see if it is signed out to the current user, and if it is,
   add this module to the message */
	modulesDir.Rewind();
	while (modulesDir.GetNextRef(&modulesRef) == B_OK)
	{
	
												// Is it a .modInfo file
		if (strstr(modulesRef.name, ".modInfo") != NULL)
		{
			BPath	moduleInfoFilePath(modulesDirPath);
			moduleInfoFilePath.Append(modulesRef.name);
			
			FILE	* moduleInfoFile = fopen(moduleInfoFilePath.Path(), "rb");
			
			char	verNoDummi[B_OS_NAME_LENGTH];
			char	status[B_OS_NAME_LENGTH];
			char	userNameTemp[B_OS_NAME_LENGTH];
			
			fscanf(moduleInfoFile, "%s", verNoDummi);
			fscanf(moduleInfoFile, "%s", status);
			fscanf(moduleInfoFile, "%s", userNameTemp);
			
			fclose(moduleInfoFile);
			
												// if it is the right user and currently signed out
			if(strcmp(user, userNameTemp) == 0 && strcmp(status, "out") == 0)
			{
												// Get the module name (which is the file name sans .zip.modInfo)
				char modNameTemp[B_FILE_NAME_LENGTH];
				strcpy(modNameTemp, modulesRef.name);
				
				char * endOfName = strstr(modNameTemp, ".zip.modInfo");
				if(endOfName != NULL)
				{
					int endIndex = endOfName - modNameTemp;
					modNameTemp[endIndex] = '\0';
												// Add this module to msg
					err = msg->AddString("module", modNameTemp);
					
					printf("Added module to mod out list: %s\n", modNameTemp);

				}
			}
		}
	}
}


// ********************************************//************
bool RepositoryLooper::IsModCheckedOut(const char * moduleName)
{
	char	modInfoFileName[B_PATH_NAME_LENGTH];
	sprintf(modInfoFileName, "%s/modules/%s.zip.modInfo", repositoryPath.Path(), moduleName);
	
	FILE	* moduleInfoFile = fopen(modInfoFileName, "rb");
	
	char	verNoDummi[B_OS_NAME_LENGTH];
	char	status[B_OS_NAME_LENGTH];
	
	fscanf(moduleInfoFile, "%s", verNoDummi);
	fscanf(moduleInfoFile, "%s", status);
	
	int		cmpVal;
	cmpVal = strcmp(status, "out");
	
	return (cmpVal == 0);
}


// ********************************************//************
status_t RepositoryLooper::CheckOut(BMessage * msg)
{
	int32	numMods;
	char	* name;
	ssize_t	size;
	status_t err;
	type_code	typeCode;

	char	* user;
	char	* passwd;

	BMessage outFiles(MIP_VC_CHECK_OUT_REPLY);
	
	err = msg->GetInfo("module", &typeCode, &numMods); //How many are in there?
	err = msg->FindString("user", &user);
	err = msg->FindString("passwd", &passwd);
	
	for (int i=0; i<numMods; i++)
	{
		msg->FindString("module", i, &name);
		err = CheckOut(name, user, &outFiles);
		if (err != B_OK)
			return err;
	}

	printf("Sending checkout message:\n");
	outFiles.PrintToStream();
	SendToClient(&outFiles);
	return B_OK;
}


// ********************************************//************
status_t RepositoryLooper::CheckOut(const char * name, const char * user, BMessage * msg)
{
	status_t err;
	
/* Parse the ModuleItem parameter and get paths and stuff */
	char modFileName[B_PATH_NAME_LENGTH];
	sprintf(modFileName, "%s/modules/%s.zip", repositoryPath.Path(), name);	
	BPath modFilePath(modFileName);
	
	char modInfoFileName[B_PATH_NAME_LENGTH];
	sprintf(modInfoFileName, "%s/modules/%s.zip.modInfo", repositoryPath.Path(), name);

	FILE * modInfoFile = fopen(modInfoFileName, "rb");
	
/* Check the current status */	
	char	ver[32], status[32];
	fscanf(modInfoFile, "%s", &ver);
	fscanf(modInfoFile, "%s", &status);
	
	fclose(modInfoFile);
	
/* Put the file in the message */
	if (strcmp(status, "in") == 0)
	{
		BPath path(modFileName);
		err = FileToStream(&path, msg, "module", "name", name);
	}
	else
		return -1;/*FIX ME!*/
	
/* Mark the info file as out */
	modInfoFile = fopen(modInfoFileName, "wb");
	
	time_t theTime = real_time_clock();

	fprintf(modInfoFile, "%s\n", ver);
	fprintf(modInfoFile, "%s\n", "out");
	fprintf(modInfoFile, "%s\n", user);
	fprintf(modInfoFile, "%d\n", theTime);
	
	fclose(modInfoFile);
	
	return err;
}


// ********************************************//************
/***** THIS ISN'T GOING TO WORK - BECAUSE ANY REPOSITORY CAN DO MULTIPLE CLIENTS *****/
status_t RepositoryLooper::SendToClient(BMessage *msg)
{
	if (remoteConnection)
	{
		status_t	status;
		NetMessenger msgr(CurrentMessage());
		if (msgr.Error() != NET_OK)
			return B_ERROR;
		status = msgr.SendMessage(msg);
		return status;
	}
	else
		return be_app->PostMessage(msg);
}


// *********************************************//************
// Fill the BMessage with modules that aren't checked out to the user
status_t RepositoryLooper::Update(const char * user)
{
	status_t	err;
	BPath		modulesDirPath(repositoryPath);
	modulesDirPath.Append("modules");
	
	BDirectory	modulesDir(modulesDirPath.Path());
	entry_ref	modulesRef;
	
	err	= modulesDir.InitCheck();
	modulesDir.Rewind();
	while (modulesDir.GetNextRef(&modulesRef) == B_OK)
	{
		if (strstr(modulesRef.name, ".modInfo") != NULL)
		{
			BPath	moduleInfoFilePath(modulesDirPath);
			moduleInfoFilePath.Append(modulesRef.name);
			
			FILE	* moduleInfoFile = fopen(moduleInfoFilePath.Path(), "rb");
			
			char	verNoTemp[B_OS_NAME_LENGTH];
			char	statusTemp[B_OS_NAME_LENGTH];
			char	userNameTemp[B_OS_NAME_LENGTH];
			time_t	lastTimeTemp;
			
			fscanf(moduleInfoFile, "%s", verNoTemp);
			fscanf(moduleInfoFile, "%s", statusTemp);
			fscanf(moduleInfoFile, "%s", userNameTemp);
//			fscanf(moduleInfoFile, "%d", lastTimeTemp);
			
			fclose(moduleInfoFile);

			ModuleItem * modItem;
			
												// If this isn't checked out to user
			
			if((strcmp(userNameTemp, user) != 0) ||
					strcmp(statusTemp, "out") != 0)
			{
												// Get the module name
				char	workModName[B_FILE_NAME_LENGTH];
				strcpy(workModName, modulesRef.name);
				char	* zipExt;
				zipExt = strstr(workModName, ".zip");
				if (zipExt != NULL)
				{
					int endIndex = zipExt - workModName;
					workModName[endIndex] = '\0';
				}
				
												// Put this module in a message & send it
				BMessage reply(MIP_VC_UPDATE_REPLY);
				
				BPath	modFilePath(modulesDirPath);
				char	modFileName[B_FILE_NAME_LENGTH];
				strcpy(modFileName, workModName);
				strcat(modFileName, ".zip");
				modFilePath.Append(modFileName);
				
				err = FileToStream(&modFilePath, &reply, "module", "name", workModName);
				if (err == B_OK)
				{
					SendToClient(&reply);
				}
			}
		}
	}
	
	return B_OK;

}





