/* File Name: ChangePasswordWindow.cpp */

/*
************************************************************************
************************************************************************

Copyright © 1998 Multimedia Integration Products, All Rights Reserved

This file is not for public release.
************************************************************************

************************************************************************
************************************************************************ */

/*
************************************************************************
************************************************************************

Copyright © 1998 Multimedia Integration Products, All Rights Reserved

This file is for public release.

This file is for reference use only. Any redistribution must include
the entire BeVC package, including source code. If you use this code
please notify j-onorato@uchicago.edu or jwhamil@students.wisc.edu

************************************************************************

************************************************************************
************************************************************************ */

#include "ChangePasswordWindow.h"
#include "SecurityManager.h"
enum
{
	CHPWD_CHANGE = 'CPw0',
	CHPWD_PICK_USER
};

#define SHADOWED_PASSWORD_STRING "••••••"

// *******************************************//************
ChangePasswordWindow::ChangePasswordWindow()
	: BWindow(BRect(100, 100, 380, 210), "Administer Local Passwords", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE )
{
	strcpy(currentUserName, "");
	
/* Layout Window */
	winView = new BView(Bounds(), "winView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	winView->SetViewColor(225, 225, 225, 255);
	
	AddChild(winView);
	
	userNameMenu = new BPopUpMenu("Pick User");
	FillUserNameMenu();
	
	userNameMenuField = new BMenuField(BRect(15, 15, 250, 35), "userName", "User Name:", userNameMenu);
	userNameMenuField->SetDivider(60);
	winView->AddChild(userNameMenuField);
	
	passwordView = new BTextControl(BRect(15, 45, 250, 90), "password", "Password:", "", NULL);
	passwordView->SetDivider(60);
	winView->AddChild(passwordView);
	
	BRect	fram(Bounds());
	fram.bottom -= 10;
	fram.top = fram.bottom - 25;
	fram.right -= 10;
	fram.left = fram.right - 70;
	
	BButton * changeButton;
	changeButton = new BButton(fram, "changeButton", "Change", new BMessage(CHPWD_CHANGE));
	winView->AddChild(changeButton);
	changeButton->MakeDefault(true);
}


// *******************************************//************
ChangePasswordWindow::~ChangePasswordWindow()
{

}

// *******************************************//************
void ChangePasswordWindow::MessageReceived(BMessage * message)
{
	switch (message->what)
	{
		case CHPWD_PICK_USER:
		{
			char * tmpUser;
			status_t err;
			
			err = message->FindString("name", &tmpUser);
			if (err != B_OK)
				return ;
			
			strncpy(currentUserName, tmpUser, B_OS_NAME_LENGTH);
			currentUserName[B_OS_NAME_LENGTH-1] = '\0';
			
			FillInInfo();
		}
		break;
	
		case CHPWD_CHANGE:
		{
			if (strcmp(currentUserName, "") ||
					strcmp(passwordView->Text(), SHADOWED_PASSWORD_STRING))
			{
				char userName[B_OS_NAME_LENGTH];
				char passwd[B_OS_NAME_LENGTH];
				
				strcpy(userName, currentUserName);
				strcpy(passwd, passwordView->Text());
				
				::ChangePassword(userName, passwd);
				if (strcmp(passwordView->Text(), ""))
					passwordView->SetText(SHADOWED_PASSWORD_STRING);
			}
		}
		break;
		default:
			inherited::MessageReceived(message);
	}
}


// *******************************************//************
bool ChangePasswordWindow::QuitRequested()
{
	if(!this->IsHidden())
		this->Hide();
	return(false);
}


// *******************************************//************
void ChangePasswordWindow::FillUserNameMenu()
{
	status_t	err;
	BPath		personDirPath;
	entry_ref	personRef;
	
	err = find_directory(B_COMMON_DIRECTORY, &personDirPath);
	if (err != B_OK)
		return;
	
	personDirPath.Append("people");
	const int32 zero = 0;
												// Remove everything from the menu
	while(userNameMenu->CountItems() != 0)
	{
		BMenuItem * menuItem = userNameMenu->RemoveItem(zero);
		if (menuItem != NULL)
			delete menuItem;
	}
	
												// Get all of the people files from the people dir
	BDirectory peopleDir(personDirPath.Path());
	
	err	= peopleDir.InitCheck();
	if (err != B_OK)
		return;
	
	peopleDir.Rewind();
	while (peopleDir.GetNextRef(&personRef) == B_OK)
	{
		BMessage	* modelMessage = new BMessage(CHPWD_PICK_USER);
		modelMessage->AddString("name", personRef.name);
		BMenuItem	* item = new BMenuItem(personRef.name, modelMessage);
		
		userNameMenu->AddItem(item);
	}
}


// *******************************************//************
void ChangePasswordWindow::FillInInfo()
{
	status_t	err;
	BPath		personPath;
	
	err = find_directory(B_COMMON_DIRECTORY, &personPath);
	if (err != B_OK)
		return;
	
	personPath.Append("people");
	personPath.Append(currentUserName);
	
	BNode personNode(personPath.Path());
	
	err = personNode.InitCheck();
	if (err != B_OK)
		return ;	

	attr_info info;
	
	err = personNode.GetAttrInfo(SM_PASSWORD_ATTR, &info);
	if ((err != B_OK) || (info.type != B_STRING_TYPE) )
	{										// Not allowed to play
		passwordView->SetText("");
		return ;
	}
											// Put something in the passwd
	passwordView->SetText(SHADOWED_PASSWORD_STRING);
}



